#include "messages.h"
#include <syslog.h>
#include <stdio.h>
#include <unistd.h>

int debug_flag = 0;
int verbose_flag = 0;
static int log_stderr = 1;
int msg_pipe[2] = { -1, -1 };

void
msg_log_func(const gchar *log_domain, GLogLevelFlags log_flags, const gchar *msg, gpointer user_data)
{
  int pri = LOG_INFO;
  gchar buf[1025];
  
  if (log_stderr || (log_flags & G_LOG_LEVEL_DEBUG))
    {
      fprintf(stderr, "%s\n", msg);
    }
  else
    {
      if (log_flags & G_LOG_LEVEL_DEBUG)
        pri = LOG_DEBUG;
      else if (log_flags & G_LOG_LEVEL_WARNING)
        pri = LOG_WARNING;
      else if (log_flags & G_LOG_LEVEL_ERROR)
        pri = LOG_ERR;
        
      pri |= LOG_SYSLOG;
      g_snprintf(buf, sizeof(buf), "<%d> syslog-ng[%d]: %s\n", pri, getpid(), msg);
      if (write(msg_pipe[1], buf, strlen(buf)) == -1)
        {
          fprintf(stderr, "%s", msg);
        }
    }
}

gboolean
msg_init(int use_stderr)
{
  if (!use_stderr)
    {
      if (pipe(msg_pipe) < 0)
        {
          msg_error("Error creating internal message pipe; error='%m'");
          return FALSE;
        }
      log_stderr = FALSE;
    }
  g_log_set_handler(G_LOG_DOMAIN, 0xff, msg_log_func, NULL);
  g_log_set_handler("GLib", 0xff, msg_log_func, NULL);
  return TRUE;
}

void
msg_deinit()
{
  close(msg_pipe[0]);
  close(msg_pipe[1]);
}
